/** 									// -o-
 * classes.pde
 *
 *   MirrorCycles
 *   GeometricIncrement
 *   ArcPointRadius
 *   FreqHalfInc
 *
 * ----------------------------------------------------------------------
 * Copyright © 2011, David Reeder.  All rights reserved.
 * Please see the LICENSE file in this distribution for license details.
 * ----------------------------------------------------------------------
 */


// ------------------------------------------------ -o--
class MirrorCycles extends SliderAction
{
  MirrorCycles(Slider so_)  { super(so_); }

  MirrorCycles triggerAction() 
  {
    float mirrorValue = slidObj.valueRangeHigh 
			  - (slidObj.value() - slidObj.valueRangeLow);
    
    slidObj.oscClearBundle();
    slidObj.oscClearMsg();

    slidObj.oscMsg( round100(slidObj.value()) );
    slidObj.oscBundle();

    slidObj.oscMsg( round100(mirrorValue) );
    slidObj.oscBundle();

    return this;
  }

}  // MirrorCycles




// ------------------------------------------------ -o--
// next() returns consecutive increments of base by increment units.
//
class GeometricIncrement 
{
  int  base, increment;
  int  value;

  GeometricIncrement() 			{ value = base; }
  GeometricIncrement(int b_, int i_) 	{ value = base = b_; increment = i_; }

  int next()  { return value += increment; }
}




// ------------------------------------------------ -o--
// next{X,Y,R}() returns circles along an arc at independent
//   increments in each dimention.  may also determine amplitude of
//   rise and/or flip phase.
//
class ArcPointRadius 
{
  int    centerX, centerY; 	// location: center of circle
  int    radius, radiusInc; 	// radius and increment for subsequent radii

  int    run; 			// location: cartesian increment to centerX
  float  rise, riseInc; 	// location: polar increment to centerY

  int    amplitude, 		// amplitude of rise
	 hflip = 1; 		// (simple) phase of rise


  ArcPointRadius( int x_, int y_, 
		  int r_, int rinc_,
		  int run_, float rise_, float riseinc_, int multi_ ) 
  {
    setXY(x_, y_);
    setRadius(r_, rinc_);
    setInc(run_, rise_, riseinc_, multi_);
  }


  void setXY(int x_, int y_) 	  	{ centerX = x_; centerY = y_; }

  void setRadius(int r_)  		{ radius = r_; radiusInc = 0; }
  void setRadius(int r_, int rinc_)	{ radius = r_; radiusInc = rinc_; }

  void setInc(int run_, float rise_, float riseinc_, int multi_) 
  {
    run = run_; rise = rise_; riseInc = riseinc_; amplitude = multi_;
  }


  void horizFlip(int hf_)  { hflip = hf_; }


  int nextX()  { return centerX += run; }
  int nextY()  {  
    return centerY += (int) hflip*(sin(rise += riseInc)*amplitude); 
  }
  int nextR()  { return radius += radiusInc; }

}  // ArcPointRadius



// ------------------------------------------------ -o--
class FreqHalfInc 	
{
  float  base, val;
  int 	 inc = 1;

  FreqHalfInc(int b_)  { val = base = b_; }

  float next()  { return val += (val/2 + inc++); }
}

